<?php
/*
	Question2Answer by Gideon Greenspan and contributors
	http://www.question2answer.org/

	File: qa-include/qa-install.php
	Description: User interface for installing, upgrading and fixing the database


	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	More about this license: http://www.question2answer.org/license.php
*/

if (!defined('QA_VERSION')) { // don't allow this page to be requested directly from browser
	header('Location: ../');
	exit;
}

require_once QA_INCLUDE_DIR.'db/install.php';

qa_report_process_stage('init_install');


// Define database failure handler for install process, if not defined already (file could be included more than once)

if (!function_exists('qa_install_db_fail_handler')) {
	/**
	 * Handler function for database failures during the installation process
	 */
	function qa_install_db_fail_handler($type, $errno=null, $error=null, $query=null)
	{
		global $pass_failure_from_install;

		$pass_failure_type = $type;
		$pass_failure_errno = $errno;
		$pass_failure_error = $error;
		$pass_failure_query = $query;
		$pass_failure_from_install = true;

		require QA_INCLUDE_DIR.'qa-install.php';

		qa_exit('error');
	}
}


$success = '';
$errorhtml = '';
$suggest = '';
$buttons = array();
$fields = array();
$fielderrors = array();
$hidden = array();


// Process user handling higher up to avoid 'headers already sent' warning

if (!isset($pass_failure_type) && qa_clicked('super')) {
	require_once QA_INCLUDE_DIR.'db/users.php';
	require_once QA_INCLUDE_DIR.'app/users-edit.php';

	$inemail = qa_post_text('email');
	$inpassword = qa_post_text('password');
	$inhandle = qa_post_text('handle');

	$fielderrors = array_merge(
		qa_handle_email_filter($inhandle, $inemail),
		qa_password_validate($inpassword)
	);

	if (empty($fielderrors)) {
		require_once QA_INCLUDE_DIR.'app/users.php';

		$userid = qa_create_new_user($inemail, $inpassword, $inhandle, QA_USER_LEVEL_SUPER);
		qa_set_logged_in_user($userid, $inhandle);

		qa_set_option('feedback_email', $inemail);
		qa_set_option('site_theme_mobile', 'SnowFlat');
		qa_set_option('site_theme', 'SnowFlat');
		qa_set_option('site_language', 'fa');
		qa_set_option('site_text_direction', 'rtl');
		
		$success .= "تبریک - وب‌سایت پرسش و پاسخ شما آماده‌ی استفاده شده است!\n\nشما به عنوان سوپرادمین وارد می‌شوید و خواهید توانست تنظیمات را تغییر دهید.\n\nاز شما به خاطر استفاده از پرسش و پاسخ فارسی متشکریم.";
	}
}


//	Output start of HTML early, so we can see a nicely-formatted list of database queries when upgrading

?><!DOCTYPE html>
<html>
	<head>
		<title>نصب کیوتوای فارسی</title>
		<meta charset="utf-8">
		<style>
table {margin: 16px auto;}
.msg-success { color: #080; }
.msg-error { color: #900; }
		</style>
<link rel="stylesheet" type="text/css" href="./install-docs/styles.css">
	</head>
	<body>
		<div style="width: 100%;">
		<div style="width: 100%; text-align: center; background:rgba(221, 221, 221, 0.8);"><a href="http://question2answer-farsi.com" target="_blank"><img class="logo" src="./install-docs/logo.png" alt="پشتیبانی فارسی اسکریپت کیوتوای" /></a>
		<p style="color:green; font-weight:normal; font-size:25px;">نگارش کوروش بزرگ</p>
		<p style="color:green; font-weight:normal; font-size:25px;">شماره 1.7.1</p>
		</div>
		
<?php


if (isset($pass_failure_type)) {
	// this page was requested due to query failure, via the fail handler
	switch ($pass_failure_type) {
		case 'connect':
			$errorhtml .= 'ناتوانی در برقراری اتصال به پایگاه داده. لطفاً در فایل تنظیمات اطلاعات مربوط به نام کاربری، رمز عبور و نام و آدرس پایگاه داده را بررسی نمایید. همچنین دسترسی‌های لازم برای کاربر این پایگاه داده را بررسی نمایید.';
			break;

		case 'select':
			$errorhtml .= 'ناتوانی در انتخاب پایگاه داده‌ی پرسش و پاسخ. لطفاً از تنظیم درست نام پایگاه داده در فایل تنظیمات مطمئن شوید. همچنین در صورت نیاز پایگاه داده‌ی مربوطه را نسب کرده و دسترسی‌های لازم برای کاربر را تعیین نمایید.';
			break;

		case 'query':
			global $pass_failure_from_install;

			if (@$pass_failure_from_install)
				$errorhtml .= "ناتوانی در اجرای اسکریپت زیر. لطفاً بررسی نمایید که کاربر مربوط به این پایگاه داده بر روی همه‌ی جداول دسترسی CREATE و ALTER را دارد:\n\n".qa_html($pass_failure_query."\n\nError ".$pass_failure_errno.": ".$pass_failure_error."\n\n");
			else
				$errorhtml .= "اجرای یک کوئری پایگاه داده با اشکال مواجه شد.\n\nتوضیحات کامل خطا در فایل لاگ سرور موجود است.";
			break;
	}
}
else {
	// this page was requested by user GET/POST, so handle any incoming clicks on buttons
	qa_db_connect('qa_install_db_fail_handler');

	if (qa_clicked('create')) {
		qa_db_install_tables();

		if (QA_FINAL_EXTERNAL_USERS) {
			if (defined('QA_FINAL_WORDPRESS_INTEGRATE_PATH')) {
				require_once QA_INCLUDE_DIR.'db/admin.php';
				require_once QA_INCLUDE_DIR.'app/format.php';

				// create link back to WordPress home page
				qa_db_page_move(qa_db_page_create(get_option('blogname'), QA_PAGE_FLAGS_EXTERNAL, get_option('home'), null, null, null), 'O', 1);

				$success .= 'کیوتوای با موفقیت بر روی دیتابیس وردپرس نصب و ساماندهی شد!';

			}
			else {
				$success .= 'پایگاه داده‌ی وب‌سایت شما برای مدیریت کاربری خارجی نصب شده است. برای تکمیل مدیریت کاربران پروژه single sign on را مطالعه نمایید.';
			}
		}
		else {
			$success .= 'پایگاه داده وبسایت شما با موفقیت نصب و ساماندهی شد';
		}
	}

	if (qa_clicked('nonuser')) {
		qa_db_install_tables();
		$success .= 'جداول اضافی مورد نیاز وب‌سایت پرسش و پاسخ نصب شده است.';
	}

	if (qa_clicked('upgrade')) {
		qa_db_upgrade_tables();
		$success .= 'پایگاه داده‌ی شما به‌روز شده است.';
	}

	if (qa_clicked('repair')) {
		qa_db_install_tables();
		$success .= 'جداول پایگاه داده‌ی شما تعمیر شده است.';
	}

	if (qa_clicked('module')) {
		$moduletype = qa_post_text('moduletype');
		$modulename = qa_post_text('modulename');

		$module = qa_load_module($moduletype, $modulename);

		$queries = $module->init_queries(qa_db_list_tables());

		if (!empty($queries)) {
			if (!is_array($queries))
				$queries = array($queries);

			foreach ($queries as $query)
				qa_db_upgrade_query($query);
		}

		$success .= 'افزونه '.$modulename.' '.$moduletype.' با موفقیت به دیتابیس اضافه شد';
	}

}

if (qa_db_connection(false) !== null && !@$pass_failure_from_install) {
	$check = qa_db_check_tables(); // see where the database is at

	switch ($check) {
		case 'none':
			if (@$pass_failure_errno == 1146) // don't show error if we're in installation process
				$errorhtml = '';

			$errorhtml .= '<div>به <a href="http://question2answer-farsi.com" target="_blank">اسکریپت کیوتوای فارسی</a> خوش آمدید. در اولین گام باید پایگاه داده را نصب نمایید.</div>';

				if (QA_FINAL_EXTERNAL_USERS) {
					if (defined('QA_FINAL_WORDPRESS_INTEGRATE_PATH'))
						$errorhtml.="\n\n<div style=\"direction: rtl;background:rgba(221, 221, 221, 0.8);padding:5px 20px;\">با کلیک بر روی دکمه‌ی زیر وب‌سایت پرسش و پاسخ شما با استفاده از جداول کاربران وردپرس موجود ساخته می‌شود. یعنی کاربران دو وب‌سایت مشترک خواهند بود. <A HREF=\"".qa_html(get_option('home'))."\" TARGET=\"_blank\">".qa_html(get_option('blogname'))."</A>. راهنما را برای اطلاعات بیشتر ببینید.</div>";
					else
						$errorhtml.="\n\n<div style=\"direction: rtl;background:rgba(221, 221, 221, 0.5);padding:5px 20px;\">با کلیک بر روی دکمه‌ی زیر وب‌سایت پرسش و پاسخ شما با استفاده از جداول کاربران موجود ساخته می‌شود، یعنی کاربران دو وب‌سایت مشترک خواهند بود. کاربران با نوع ستون پایگاه داده‌ی ".qa_html(qa_get_mysql_user_column_type())." مورد اشاره قرار خواهند گرفت. برای اطلاعات بیشتر راهنما را ببینید.</div>";
					
					$buttons=array('create' => 'نصب پایگاه داده');
				} else {
					$errorhtml.="\n\n<div style=\"direction: rtl;background:rgba(221, 221, 221, 0.8);padding:5px 20px;\">با کلیک بر روی دکمه‌ی زیر پایگاه داده به صورت مستقل نصب می‌شود و اطلاعات کاربران در داخل آن قرار می‌گیرد.\n\nتوجه : اگر مایل به استفاده از کاربران وب‌سایت مشابه دیگری هستید، یعنی می‌خواهید کاربران دو اسکریپت مشترک باشند، ابتدا پروژه single sign on را در بخش آموزش های سایت مطالعه نمایید , همچنین این اسکریپت امکان نصب بر روی وردپرس را دارا می باشد برای اطلاعات بیشتر به وبسایت کیوتوای فارسی مراجعه نمایید.</div>";
					$buttons=array('create' => 'آغاز عملیات نصب پایگاه داده اسکریپت');
				}
				break;

		case 'old-version':
			// don't show error if we need to upgrade
			if (!@$pass_failure_from_install)
				$errorhtml = '';

				$errorhtml.='<div style=\"direction: rtl;background:rgba(221, 221, 221, 0.8);padding:5px 20px;\">پایگاه داده‌ی شما باید برای این نسخه از کیوتوای به‌روز شود.,/div>'; // don't show error before this
				$buttons=array('upgrade' => 'به‌روزرسانی پایگاه داده');
				break;
				
			case 'non-users-missing':
				$errorhtml='این وب‌سایت پرسش و پاسخ کاربران را با وب‌سایت Q2A دیگری به اشتراک گذاشته است، اما برای اطلاعات مربوط به خود وب‌سایت جداول دیگری مورد نیاز است. برای ساخت آن‌ها کلیک نمایید.';
				$buttons=array('nonuser' => 'ساخت جداول');
				break;
				
			case 'table-missing':
				$errorhtml.='چند جدول در پایگاه داده‌ی وب‌سایت شما ناموجود است.';
				$buttons=array('repair' => 'تعمیر پایگاه داده');
				break;
				
			case 'column-missing':
				$errorhtml.='چند جدول فاقد ستون‌های لازم هستند.';
				$buttons=array('repair' => 'تعمیر پایگاه داده');
				break;

		default:
			require_once QA_INCLUDE_DIR.'db/admin.php';

			if (!QA_FINAL_EXTERNAL_USERS && qa_db_count_users() == 0) {
				$errorhtml .= "هنوز هیچ کاربری در پایگاه داده موجود نیست.\n\nبرای ساخت مدیریت کل اطلاعات خود را وارد نمایید:";
				$fields = array(
					'handle' => array('label' => 'نام کاربری : ', 'type' => 'text'),
					'password' => array('label' => 'رمز عبور : ', 'type' => 'password'),
					'email' => array('label' => 'ایمیل : ', 'type' => 'text'),
				);
				$buttons = array('super' => 'ساخت حساب مدیریت کل');
			}
			else {
				$tables = qa_db_list_tables();

				$moduletypes = qa_list_module_types();

				foreach ($moduletypes as $moduletype) {
					$modules = qa_load_modules_with($moduletype, 'init_queries');

					foreach ($modules as $modulename => $module) {
						$queries = $module->init_queries($tables);
						if (!empty($queries)) {
							// also allows single query to be returned
							$errorhtml = strtr(qa_lang_html('admin/module_x_database_init'), array(
								'^1' => qa_html($modulename),
								'^2' => qa_html($moduletype),
								'^3' => '',
								'^4' => '',
							));

							$buttons = array('module' => 'مقدار دهی به پایگاه داده');

							$hidden['moduletype'] = $moduletype;
							$hidden['modulename'] = $modulename;
							break;
						}
					}
				}
			}
			break;
	}
}

if (empty($errorhtml)) {
	if (empty($success))
		$success = 'پایگاه داده‌ی شما با موفقیت بررسی شد و خطایی ندارد.';

	$suggest = '<a href="'.qa_path_html('admin', null, null, QA_URL_FORMAT_SAFEST).'">بازگشایی صفحه مدیریت</a>';
}

?>

		<form method="post" action="<?php echo qa_path_html('install', null, null, QA_URL_FORMAT_SAFEST)?>">

<?php

if (strlen($success))
	echo '<p class="msg-success">'.nl2br(qa_html($success)).'</p>';

if (strlen($errorhtml))
	echo '<p class="msg-error">'.nl2br($errorhtml).'</p>';

if (strlen($suggest))
	echo '<p>'.$suggest.'</p>';


//	Very simple general form display logic (we don't use theme since it depends on tons of DB options)

if (count($fields)) {
	echo '<table>';

	foreach ($fields as $name => $field) {
		echo '<tr>';
		echo '<th>'.qa_html($field['label']).'</th>';
		echo '<td><input type="'.qa_html($field['type']).'" size="24" name="'.qa_html($name).'" value="'.qa_html(@${'in'.$name}).'"></td>';
		if (isset($fielderrors[$name]))
			echo '<td class="msg-error"><small>'.qa_html($fielderrors[$name]).'</small></td>';
		else
			echo '<td></td>';
		echo '</tr>';
	}

	echo '</table>';
}

foreach ($buttons as $name => $value)
	echo '<input type="submit" name="'.qa_html($name).'" value="'.qa_html($value).'">';

foreach ($hidden as $name => $value)
	echo '<input type="hidden" name="'.qa_html($name).'" value="'.qa_html($value).'">';

qa_db_disconnect();

?>

		</form>
	</body>
</html>
